package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.mapper.ReferenceMaterialMapper;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.service.StandardMaterialApprovalFormService;
import digital.laboratory.platform.reagent.vo.ReferenceMaterialVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * 服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-23
 * @describe 服务实现类
 */

@Service
@SuppressWarnings("all")
public class ReferenceMaterialServiceImpl extends ServiceImpl<ReferenceMaterialMapper, ReferenceMaterial> implements ReferenceMaterialService {


    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private StandardMaterialApprovalFormService standardMaterialApprovalFormService;

    /**
     * 通过批次ID查询标准物质
     *
     * @param batchDetailsId
     * @return
     */
    @Override
    public List<ReferenceMaterialVO> getReferenceMaterialVOList(String batchDetailsId) {
        LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();
        referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getBatchDetailsId, batchDetailsId);
        List<ReferenceMaterial> list = this.list(referenceMaterialLambdaQueryWrapper);
        List<ReferenceMaterialVO> referenceMaterialVOS = new ArrayList<>();
        for (ReferenceMaterial referenceMaterial : list) {
            ReagentConsumables byId = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());
            ReferenceMaterialVO referenceMaterialVO = new ReferenceMaterialVO();
            BeanUtils.copyProperties(referenceMaterial, referenceMaterialVO);
            referenceMaterialVO.setReferenceMaterialName(byId.getReagentConsumableName());
            referenceMaterialVOS.add(referenceMaterialVO);
        }

        return referenceMaterialVOS;
    }

    @Override
    public List<ReferenceMaterialVO> getReferenceMaterialList() {
        List<ReferenceMaterialVO> referenceMaterialVOList = baseMapper.getReferenceMaterialVOList();
        return referenceMaterialVOList;
    }

    @Override
    public String createNo(String alias, String location) {
        int year = Calendar.getInstance().get(Calendar.YEAR);//获取当前年
        String prefix = alias + "-" + year + "-" + location.charAt(1) + "-";
        List<ReferenceMaterial> list = this.list(Wrappers.<ReferenceMaterial>query()
                .likeRight("number", prefix)
                .orderByDesc("number"));
        int newNo = 1;
        if ((list != null) && (list.size() > 0)) {
            ReferenceMaterial referenceMaterial1 = list.get(0);
            String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
            try {
                int maxno = Integer.parseUnsignedInt(strMaxNo);
                newNo = maxno + 1;
            } catch (NumberFormatException e) {
                // 如果后缀有非数字, 则无视之, 重头编码
                newNo = 1;
            }
        }
        return prefix + newNo;
    }


}
